@@ -2,19 +2,8 @@
 
 """Copyright (C) 2009-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-Kajongg is free software you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation either version 2 of the License, or
-(at your option) any later version.
+SPDX-License-Identifier: GPL-2.0
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 
 
@@ -31,7 +20,7 @@ from tilesource import TileSource
 from meld import Meld, MeldList
 from rule import Score, UsedRule
 from common import Debug, StrMixin
-from intelligence import AIDefault
+from intelligence import AIDefaultAI
 from util import callers
 from message import Message
 
@@ -96,7 +85,7 @@ class Hand(StrMixin):
         # shortcuts for speed:
         self._player = weakref.ref(player)
         self.ruleset = player.game.ruleset
-        self.intelligence = player.intelligence if player else AIDefault()
+        self.intelligence = player.intelligence if player else AIDefaultAI()
         self.string = string
         self.__robbedTile = Tile.unknown
         self.prevHand = prevHand
@@ -124,7 +113,7 @@ class Hand(StrMixin):
             self.debug(fmt('{callers}',
                            callers=callers(exclude=['__init__'])))
             Hand.indent += 1
-            self.debug('New Hand {} {}'.format(string, self.lenOffset))
+            self.debug('New Hand {} lenOffset={}'.format(string, self.lenOffset))
 
         try:
             self.__arrange()
@@ -138,7 +127,7 @@ class Hand(StrMixin):
         finally:
             self._fixed = True
             if Debug.hand or (Debug.mahJongg and self.lenOffset == 1):
-                self.debug('Fixing {} {} {}'.format(self, self.won, self.score))
+                self.debug('Fixing {} {}{}'.format(self, 'won ' if self.won else '', self.score))
             Hand.indent -= 1
 
     def __parseString(self, inString):
@@ -151,8 +140,8 @@ class Hand(StrMixin):
                 if len(part) > 1:
                     try:
                         self.__lastSource = TileSource.byChar[part[1]]
-                    except KeyError:
-                        raise Exception('{} has unknown lastTile {}'.format(inString, part[1]))
+                    except KeyError as _:
+                        raise Exception('{} has unknown lastTile {}'.format(inString, part[1])) from _
                     if len(part) > 2:
                         self.__announcements = set(part[2])
             elif partId == 'L':
@@ -175,7 +164,7 @@ class Hand(StrMixin):
                 tileStrings.remove(part)
 
         self.values = tuple(x.value for x in self.tiles)
-        self.suits = set(x.lowerGroup for x in self.tiles)
+        self.suits = {x.lowerGroup for x in self.tiles}
         self.declaredMelds = MeldList(x for x in self.melds if x.isDeclared)
         declaredTiles = list(sum((x for x in self.declaredMelds), []))
         self.tilesInHand = TileList(x for x in self.tiles
@@ -185,7 +174,7 @@ class Hand(StrMixin):
 
         assert len(tileStrings) < 2, tileStrings
         self.__rest = TileList()
-        if len(tileStrings):
+        if tileStrings:
             self.__rest.extend(TileList(tileStrings[0][1:]))
 
         last = self.__lastTile
@@ -329,22 +318,19 @@ class Hand(StrMixin):
         else:  # not self.won
             loserRules = self.__matchingRules(self.ruleset.loserRules)
             if loserRules:
-                self.usedRules.extend(list(UsedRule(x) for x in loserRules))
+                self.usedRules.extend(UsedRule(x) for x in loserRules)
                 self.__score = self.__totalScore()
         self.__checkHasExclusiveRules()
 
     def matchingWinnerRules(self):
-        """returns a list of matching winner rules"""
-        matching = list(
-            UsedRule(x)
-            for x in self.__matchingRules(self.ruleset.winnerRules))
+        """return a list of matching winner rules"""
+        matching = [UsedRule(x) for x in self.__matchingRules(self.ruleset.winnerRules)]
         limitRule = self.maxLimitRule(matching)
         return [limitRule] if limitRule else matching
 
     def __checkHasExclusiveRules(self):
         """if we have one, remove all others"""
-        exclusive = list(x for x in self.usedRules
-                         if 'absolute' in x.rule.options)
+        exclusive = [x for x in self.usedRules if 'absolute' in x.rule.options]
         if exclusive:
             self.usedRules = exclusive
             self.__score = self.__totalScore()
@@ -352,7 +338,7 @@ class Hand(StrMixin):
                 raise Hand.__NotWon(fmt('exclusive rule {exclusive} does not win'))
 
     def __setLastMeld(self):
-        """sets the shortest possible last meld. This is
+        """set the shortest possible last meld. This is
         not yet the final choice, see __applyBestLastMeld"""
         self.__lastMeld = None
         if self.lastTile and self.__won:
@@ -389,7 +375,7 @@ class Hand(StrMixin):
         if totals:
             totals = sorted(totals)  # sort by totalScore
             maxScore = totals[-1][0]
-            totals = list(x[1] for x in totals if x[0] == maxScore)
+            totals = [x[1] for x in totals if x[0] == maxScore]
             # now we have a list of only lastMelds reaching maximum score
             if prev not in totals or self.__lastMeld not in totals:
                 if Debug.explain and prev not in totals:
@@ -424,7 +410,7 @@ class Hand(StrMixin):
             lastTile = self.lastTile
         if lastMeld == 1:
             lastMeld = self.__lastMeld
-        parts = list(str(x) for x in sorted(melds))
+        parts = [str(x) for x in sorted(melds)]
         if rest:
             parts.append('R' + ''.join(str(x) for x in sorted(rest)))
         if lastSource or announcements:
@@ -436,7 +422,7 @@ class Hand(StrMixin):
         return ' '.join(parts).strip()
 
     def __add__(self, addTile):
-        """returns a new Hand built from this one plus addTile"""
+        """return a new Hand built from this one plus addTile"""
         assert addTile.isConcealed, 'addTile %s should be concealed:' % addTile
         # combine all parts about hidden tiles plus the new one to one part
         # because something like DrDrS8S9 plus S7 will have to be reordered
@@ -451,7 +437,7 @@ class Hand(StrMixin):
         return Hand(self.player, newString, prevHand=self)
 
     def __sub__(self, subtractTile):
-        """returns a copy of self minus subtractTiles.
+        """return a copy of self minus subtractTiles.
         Case of subtractTile (hidden or exposed) is ignored.
         subtractTile must either be undeclared or part of
         lastMeld. Exposed melds of length<3 will be hidden."""
@@ -492,7 +478,7 @@ class Hand(StrMixin):
         return Hand(self.player, newString, prevHand=self)
 
     def manualRuleMayApply(self, rule):
-        """returns True if rule has selectable() and applies to this hand"""
+        """return True if rule has selectable() and applies to this hand"""
         if self.__won and rule in self.ruleset.loserRules:
             return False
         if not self.__won and rule in self.ruleset.winnerRules:
@@ -532,7 +518,7 @@ class Hand(StrMixin):
             if hand.won:
                 result.append(hand)
         if Debug.hand:
-            _hiderules = ', '.join(set(x.mjRule.name for x in result))
+            _hiderules = ', '.join({x.mjRule.name for x in result})
             if _hiderules:
                 self.debug(fmt('Is calling {_hiderules}'))
         return result
@@ -570,6 +556,7 @@ class Hand(StrMixin):
                     self.debug(fmt('{callers} Found {matchingMJRules}',
                                    callers=callers()))
                 return result
+        return None
 
     def __arrangements(self):
         """find all legal arrangements.
@@ -656,11 +643,10 @@ class Hand(StrMixin):
             return True
         if self.won and not (other.arranged and other.won):
             return True
-        elif not (self.arranged and self.won) and other.won:
+        if not (self.arranged and self.won) and other.won:
             return False
-        else:
-            return (self.intelligence.handValue(self)
-                    > self.intelligence.handValue(other))
+        return (self.intelligence.handValue(self)
+                > self.intelligence.handValue(other))
 
     def __lt__(self, other):
         """compares hand values"""
@@ -678,14 +664,14 @@ class Hand(StrMixin):
 
     def __matchingRules(self, rules):
         """return all matching rules for this hand"""
-        return list(rule for rule in rules if rule.appliesToHand(self))
+        return [rule for rule in rules if rule.appliesToHand(self)]
 
     @staticmethod
     def maxLimitRule(usedRules):
-        """returns the rule with the highest limit score or None"""
+        """return the rule with the highest limit score or None"""
         result = None
         maxLimit = 0
-        usedRules = list(x for x in usedRules if x.rule.score.limits)
+        usedRules = [x for x in usedRules if x.rule.score.limits]
         for usedRule in usedRules:
             score = usedRule.rule.score
             if score.limits > maxLimit:
